@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2012 Conrad Sanderson
-// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2014 Conrad Sanderson
+// Copyright (C) 2008-2014 NICTA (www.nicta.com.au)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -96,7 +96,8 @@ arma_vec_norm_2
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  typedef typename T1::pod_type T;
+  typedef typename T1::elem_type eT;
+  typedef typename T1::pod_type   T;
   
   T acc = T(0);
   
@@ -167,7 +168,71 @@ arma_vec_norm_2
       }
     }
   
-  return std::sqrt(acc);
+  
+  const T sqrt_acc = std::sqrt(acc);
+  
+  if( (sqrt_acc != T(0)) && arma_isfinite(sqrt_acc) )
+    {
+    return sqrt_acc;
+    }
+  else
+    {
+    arma_extra_debug_print("arma_vec_norm_2(): detected possible underflow or overflow");
+  
+    const quasi_unwrap<typename Proxy<T1>::stored_type> R(P.Q);
+    
+    const uword N     = R.M.n_elem;
+    const eT*   R_mem = R.M.memptr();
+    
+    eT max_val = priv::most_neg<eT>();
+    
+    uword i,j;
+    
+    for(i=0, j=1; j<N; i+=2, j+=2)
+      {
+      eT val_i = R_mem[i];
+      eT val_j = R_mem[j];
+      
+      val_i = std::abs(val_i);
+      val_j = std::abs(val_j);
+      
+      if(val_i > max_val)  { max_val = val_i; }
+      if(val_j > max_val)  { max_val = val_j; }
+      }
+    
+    if(i < N)
+      {
+      const eT val_i = std::abs(R_mem[i]);
+      
+      if(val_i > max_val)  { max_val = val_i; }
+      }
+    
+    if(max_val == eT(0))  { return eT(0); }
+    
+    eT alt_acc1 = eT(0);
+    eT alt_acc2 = eT(0);
+    
+    for(i=0, j=1; j<N; i+=2, j+=2)
+      {
+      eT val_i = R_mem[i];
+      eT val_j = R_mem[j];
+      
+      val_i /= max_val;
+      val_j /= max_val;
+      
+      alt_acc1 += val_i * val_i;
+      alt_acc2 += val_j * val_j;
+      }
+    
+    if(i < N)
+      {
+      const eT val_i = R_mem[i] / max_val;
+      
+      alt_acc1 += val_i*val_i;
+      }
+    
+    return ( std::sqrt(alt_acc1 + alt_acc2) * max_val ); 
+    }
   }
 
 
@@ -185,7 +250,8 @@ arma_vec_norm_2
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  typedef typename T1::pod_type T;
+  typedef typename T1::elem_type eT;
+  typedef typename T1::pod_type   T;
   
   T acc = T(0);
   
@@ -225,7 +291,43 @@ arma_vec_norm_2
       }
     }
   
-  return std::sqrt(acc);
+  const T sqrt_acc = std::sqrt(acc);
+  
+  if( (sqrt_acc != T(0)) && arma_isfinite(sqrt_acc) )
+    {
+    return sqrt_acc;
+    }
+  else
+    {
+    arma_extra_debug_print("arma_vec_norm_2(): detected possible underflow or overflow");
+    
+    const quasi_unwrap<typename Proxy<T1>::stored_type> R(P.Q);
+    
+    const uword N     = R.M.n_elem;
+    const eT*   R_mem = R.M.memptr();
+    
+    T max_val = priv::most_neg<T>();
+    
+    for(uword i=0; i<N; ++i)
+      {
+      const T val_i = std::abs(R_mem[i]);
+      
+      if(val_i > max_val)  { max_val = val_i; }
+      }
+    
+    if(max_val == T(0))  { return T(0); }
+    
+    T alt_acc = T(0);
+    
+    for(uword i=0; i<N; ++i)
+      {
+      const T val_i = std::abs(R_mem[i]) / max_val;
+      
+      alt_acc += val_i * val_i;
+      }
+    
+    return ( std::sqrt(alt_acc) * max_val ); 
+    }
   }
 
 
@@ -476,7 +578,7 @@ typename enable_if2< is_arma_type<T1>::value, typename T1::pod_type >::result
 norm
   (
   const T1& X,
-  const uword k,
+  const uword k = uword(2),
   const typename arma_real_or_cx_only<typename T1::elem_type>::result* junk = 0
   )
   {
@@ -603,212 +705,4 @@ norm
 
 
 
-//
-// norms for sparse matrices
-
-
-
-template<typename T1>
-inline
-typename T1::pod_type
-arma_mat_norm_1(const SpProxy<T1>& P)
-  {
-  arma_extra_debug_sigprint();
-  
-  // TODO: this can be sped up with a dedicated implementation
-  return as_scalar( max( sum(abs(P.Q), 0), 1) );
-  }
-
-
-
-// template<typename T1>
-// inline
-// typename T1::pod_type
-// arma_mat_norm_2(const SpProxy<T1>& P)
-//   {
-//   arma_extra_debug_sigprint();
-//   
-//   // TODO: norm = sqrt( largest eigenvalue of (A^H)*A ), where ^H is the conjugate transpose
-//   // TODO: can use ARPACK or directly implement the Arnoldi iteration
-//   // http://math.stackexchange.com/questions/4368/computing-the-largest-eigenvalue-of-a-very-large-sparse-matrix
-//   }
-
-
-
-template<typename T1>
-inline
-typename T1::pod_type
-arma_mat_norm_inf(const SpProxy<T1>& P)
-  {
-  arma_extra_debug_sigprint();
-  
-  // TODO: this can be sped up with a dedicated implementation
-  return as_scalar( max( sum(abs(P.Q), 1), 0) );
-  }
-
-
-
-template<typename T1>
-inline
-arma_warn_unused
-typename enable_if2< is_arma_sparse_type<T1>::value, typename T1::pod_type >::result
-norm
-  (
-  const T1& X,
-  const uword k,
-  const typename arma_real_or_cx_only<typename T1::elem_type>::result* junk = 0
-  )
-  {
-  arma_extra_debug_sigprint();
-  arma_ignore(junk);
-  
-  typedef typename T1::elem_type eT;
-  typedef typename T1::pod_type   T;
-  
-  const SpProxy<T1> P(X);
-  
-  if(P.get_n_nonzero() == 0)
-    {
-    return T(0);
-    }
-  
-  const bool is_vec = (P.get_n_rows() == 1) || (P.get_n_cols() == 1);
-  
-  if(is_vec == true)
-    {
-    const unwrap_spmat<typename SpProxy<T1>::stored_type> tmp(P.Q);
-    const SpMat<eT>& A = tmp.M;
-    
-    // create a fake dense vector to allow reuse of code for dense vectors
-    Col<eT> fake_vector( access::rwp(A.values), A.n_nonzero, false );
-    
-    const Proxy< Col<eT> > P_fake_vector(fake_vector);
-    
-    switch(k)
-      {
-      case 1:
-        return arma_vec_norm_1(P_fake_vector);
-        break;
-      
-      case 2:
-        return arma_vec_norm_2(P_fake_vector);
-        break;
-      
-      default:
-        {
-        arma_debug_check( (k == 0), "norm(): k must be greater than zero"   );
-        return arma_vec_norm_k(P_fake_vector, int(k));
-        }
-      }
-    }
-  else
-    {
-    switch(k)
-      {
-      case 1:
-        return arma_mat_norm_1(P);
-        break;
-      
-      // case 2:
-      //   return arma_mat_norm_2(P);
-      //   break;
-      
-      default:
-        arma_stop("norm(): unsupported or unimplemented norm type for sparse matrices");
-        return T(0);
-      }
-    }
-  }
-
-
-
-template<typename T1>
-inline
-arma_warn_unused
-typename enable_if2< is_arma_sparse_type<T1>::value, typename T1::pod_type >::result
-norm
-  (
-  const T1& X,
-  const char* method,
-  const typename arma_real_or_cx_only<typename T1::elem_type>::result* junk = 0
-  )
-  {
-  arma_extra_debug_sigprint();
-  arma_ignore(junk);
-  
-  typedef typename T1::elem_type eT;
-  typedef typename T1::pod_type   T;
-  
-  const SpProxy<T1> P(X);
-  
-  if(P.get_n_nonzero() == 0)
-    {
-    return T(0);
-    }
-  
-  
-  const unwrap_spmat<typename SpProxy<T1>::stored_type> tmp(P.Q);
-  const SpMat<eT>& A = tmp.M;
-  
-  // create a fake dense vector to allow reuse of code for dense vectors
-  Col<eT> fake_vector( access::rwp(A.values), A.n_nonzero, false );
-  
-  const Proxy< Col<eT> > P_fake_vector(fake_vector);
-  
-  
-  const char sig    = (method != NULL) ? method[0] : char(0);
-  const bool is_vec = (P.get_n_rows() == 1) || (P.get_n_cols() == 1);
-  
-  if(is_vec == true)
-    {
-    if( (sig == 'i') || (sig == 'I') || (sig == '+') )   // max norm
-      {
-      return arma_vec_norm_max(P_fake_vector);
-      }
-    else
-    if(sig == '-')   // min norm
-      {
-      const T val = arma_vec_norm_min(P_fake_vector);
-      
-      if( P.get_n_nonzero() < P.get_n_elem() )
-        {
-        return (std::min)(T(0), val);
-        }
-      else
-        {
-        return val;
-        }
-      }
-    else
-    if( (sig == 'f') || (sig == 'F') )
-      {
-      return arma_vec_norm_2(P_fake_vector);
-      }
-    else
-      {
-      arma_stop("norm(): unsupported vector norm type");
-      return T(0);
-      }
-    }
-  else
-    {
-    if( (sig == 'i') || (sig == 'I') || (sig == '+') )   // inf norm
-      {
-      return arma_mat_norm_inf(P);
-      }
-    else
-    if( (sig == 'f') || (sig == 'F') )
-      {
-      return arma_vec_norm_2(P_fake_vector);
-      }
-    else
-      {
-      arma_stop("norm(): unsupported matrix norm type");
-      return T(0);
-      }
-    }
-  }
-
-
-
 //! @}
