@@ -1,14 +1,9 @@
-// Copyright (C) 2011 NICTA (www.nicta.com.au)
-// Copyright (C) 2011 Conrad Sanderson
+// Copyright (C) 2011-2012 Conrad Sanderson
+// Copyright (C) 2011-2012 NICTA (www.nicta.com.au)
 // 
-// This file is part of the Armadillo C++ library.
-// It is provided without any warranty of fitness
-// for any purpose. You can redistribute this file
-// and/or modify it under the terms of the GNU
-// Lesser General Public License (LGPL) as published
-// by the Free Software Foundation, either version 3
-// of the License or (at your option) any later version.
-// (see http://www.opensource.org/licenses for more info)
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 
 //! \addtogroup fn_syl_lyap
@@ -49,7 +44,7 @@ syl
     arma_bad("syl(): equation appears to be singular", false);
     }
   
-  return false;
+  return status;
   }
 
 
@@ -70,8 +65,17 @@ syl
   
   typedef typename T1::elem_type eT;
   
+  const unwrap<T1> tmp_A( in_A.get_ref() );
+  const unwrap<T2> tmp_B( in_B.get_ref() );
+  const unwrap<T3> tmp_C( in_C.get_ref() );
+  
+  const Mat<eT>& A = tmp_A.M;
+  const Mat<eT>& B = tmp_B.M;
+  const Mat<eT>& C = tmp_C.M;
+  
   Mat<eT> out;
-  const bool status = syl(out, in_A, in_B, in_C);
+  
+  const bool status = auxlib::syl(out, A, B, C);
   
   if(status == false)
     {
