@@ -66,6 +66,7 @@ class op_diagvec;
 class op_vectorise_col;
 class op_normalise_colvec;
 class op_normalise_rowvec;
+class op_clamp;
 
 class eop_conj;
 
@@ -78,6 +79,8 @@ class glue_rel_lteq;
 class glue_rel_gteq;
 class glue_rel_eq;
 class glue_rel_noteq;
+class glue_rel_and;
+class glue_rel_or;
 
 class op_rel_lt_pre;
 class op_rel_lt_post;
