@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2013 Conrad Sanderson
-// Copyright (C) 2008-2013 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2014 Conrad Sanderson
+// Copyright (C) 2008-2014 NICTA (www.nicta.com.au)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -463,6 +463,7 @@ struct unwrap_check< Mat<eT> >
     arma_extra_debug_sigprint();
     }
   
+  inline
   unwrap_check(const Mat<eT>& A, const bool is_alias)
     : M_local( is_alias ? new Mat<eT>(A) : 0 )
     , M      ( is_alias ? (*M_local)     : A )
@@ -499,6 +500,7 @@ struct unwrap_check< Row<eT> >
     arma_extra_debug_sigprint();
     }
   
+  inline
   unwrap_check(const Row<eT>& A, const bool is_alias)
     : M_local( is_alias ? new Row<eT>(A) : 0 )
     , M      ( is_alias ? (*M_local)     : A )
@@ -535,6 +537,7 @@ struct unwrap_check< Col<eT> >
     arma_extra_debug_sigprint();
     }
   
+  inline
   unwrap_check(const Col<eT>& A, const bool is_alias)
     : M_local( is_alias ? new Col<eT>(A) : 0 )
     , M      ( is_alias ? (*M_local)     : A )
@@ -577,7 +580,7 @@ struct unwrap_check_mixed
     arma_extra_debug_sigprint();
     }
   
-  template<typename eT2>
+  //template<typename eT2>
   inline
   unwrap_check_mixed(const T1& A, const bool)
     : M(A)
@@ -602,7 +605,7 @@ struct unwrap_check_mixed< Mat<eT1> >
     arma_extra_debug_sigprint();
     }
   
-  template<typename eT2>
+  //template<typename eT2>
   inline
   unwrap_check_mixed(const Mat<eT1>& A, const bool is_alias)
     : M_local( is_alias ? new Mat<eT1>(A) : 0 )
@@ -640,7 +643,7 @@ struct unwrap_check_mixed< Row<eT1> >
     }
   
   
-  template<typename eT2>
+  //template<typename eT2>
   inline
   unwrap_check_mixed(const Row<eT1>& A, const bool is_alias)
     : M_local( is_alias ? new Row<eT1>(A) : 0 )
@@ -677,7 +680,7 @@ struct unwrap_check_mixed< Col<eT1> >
     arma_extra_debug_sigprint();
     }
   
-  template<typename eT2>
+  //template<typename eT2>
   inline
   unwrap_check_mixed(const Col<eT1>& A, const bool is_alias)
     : M_local( is_alias ? new Col<eT1>(A) : 0 )
@@ -723,6 +726,8 @@ struct partial_unwrap_default
   
   arma_hot arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
   static const bool do_trans = false;
   static const bool do_times = false;
   
@@ -745,6 +750,8 @@ struct partial_unwrap_fixed
   
   arma_hot arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = false;
   
@@ -788,6 +795,8 @@ struct partial_unwrap< Mat<eT> >
   
   inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return ((&X) == (&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = false;
   
@@ -810,6 +819,8 @@ struct partial_unwrap< Row<eT> >
   
   inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = false;
   
@@ -832,6 +843,8 @@ struct partial_unwrap< Col<eT> >
   
   inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = false;
   
@@ -840,8 +853,6 @@ struct partial_unwrap< Col<eT> >
 
 
 
-// NOTE: we can get away with this shortcut as the partial_unwrap class is only used by as_scalar(),
-// NOTE: which doesn't need to check for aliasing (ie. it doesn't need to compare the address of M with another matrix)
 template<typename eT>
 struct partial_unwrap< subview_col<eT> >
   {
@@ -849,17 +860,45 @@ struct partial_unwrap< subview_col<eT> >
   
   inline
   partial_unwrap(const subview_col<eT>& A)
-    : M  ( const_cast<eT*>( A.colptr(0) ), A.n_rows, false, false )
+    : orig( A.m )
+    , M   ( const_cast<eT*>( A.colptr(0) ), A.n_rows, false, false )
     {
     arma_extra_debug_sigprint();
     }
   
   inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&orig)); }
+  
   static const bool do_trans = false;
   static const bool do_times = false;
   
-  const Col<eT> M;
+  const Mat<eT>& orig;
+  const Col<eT>  M;
+  };
+
+
+
+template<typename eT>
+struct partial_unwrap< subview_row<eT> >
+  {
+  typedef Row<eT> stored_type;
+  
+  inline
+  partial_unwrap(const subview_row<eT>& A)
+    : M(A)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  inline eT get_val() const { return eT(1); }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
+  static const bool do_trans = false;
+  static const bool do_times = false;
+  
+  const Row<eT> M;
   };
 
 
@@ -879,6 +918,8 @@ struct partial_unwrap_htrans_default
   
   arma_hot arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
   static const bool do_trans = true;
   static const bool do_times = false;
   
@@ -901,6 +942,8 @@ struct partial_unwrap_htrans_fixed
   
   arma_hot arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = false;
   
@@ -943,6 +986,8 @@ struct partial_unwrap< Op< Mat<eT>, op_htrans> >
   
   arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = false;
   
@@ -965,6 +1010,8 @@ struct partial_unwrap< Op< Row<eT>, op_htrans> >
   
   arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = false;
   
@@ -987,6 +1034,8 @@ struct partial_unwrap< Op< Col<eT>, op_htrans> >
   
   arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = false;
   
@@ -995,8 +1044,6 @@ struct partial_unwrap< Op< Col<eT>, op_htrans> >
 
 
 
-// NOTE: we can get away with this shortcut as the partial_unwrap class is only used by as_scalar(),
-// NOTE: which doesn't need to check for aliasing (ie. it doesn't need to compare the address of M with another matrix)
 template<typename eT>
 struct partial_unwrap< Op< subview_col<eT>, op_htrans> >
   {
@@ -1004,17 +1051,45 @@ struct partial_unwrap< Op< subview_col<eT>, op_htrans> >
   
   inline
   partial_unwrap(const Op< subview_col<eT>, op_htrans>& A)
-    : M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, false, false )
+    : orig( A.m.m )
+    , M   ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, false, false )
     {
     arma_extra_debug_sigprint();
     }
   
   arma_inline eT get_val() const { return eT(1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&orig)); }
+  
   static const bool do_trans = true;
   static const bool do_times = false;
   
-  const Col<eT> M;
+  const Mat<eT>& orig;
+  const Col<eT>  M;
+  };
+
+
+
+template<typename eT>
+struct partial_unwrap< Op< subview_row<eT>, op_htrans> >
+  {
+  typedef Row<eT> stored_type;
+  
+  inline
+  partial_unwrap(const Op< subview_row<eT>, op_htrans>& A)
+    : M(A.m)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_inline eT get_val() const { return eT(1); }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
+  static const bool do_trans = true;
+  static const bool do_times = false;
+  
+  const Row<eT> M;
   };
 
 
@@ -1035,6 +1110,8 @@ struct partial_unwrap_htrans2_default
   
   arma_inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
   static const bool do_trans = true;
   static const bool do_times = true;
   
@@ -1059,6 +1136,8 @@ struct partial_unwrap_htrans2_fixed
   
   arma_inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = true;
   
@@ -1103,6 +1182,8 @@ struct partial_unwrap< Op< Mat<eT>, op_htrans2> >
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = true;
   
@@ -1127,6 +1208,8 @@ struct partial_unwrap< Op< Row<eT>, op_htrans2> >
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = true;
   
@@ -1151,6 +1234,8 @@ struct partial_unwrap< Op< Col<eT>, op_htrans2> >
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = true;
   static const bool do_times = true;
   
@@ -1160,8 +1245,6 @@ struct partial_unwrap< Op< Col<eT>, op_htrans2> >
 
 
 
-// NOTE: we can get away with this shortcut as the partial_unwrap class is only used by as_scalar(),
-// NOTE: which doesn't need to check for aliasing (ie. it doesn't need to compare the address of M with another matrix)
 template<typename eT>
 struct partial_unwrap< Op< subview_col<eT>, op_htrans2> >
   {
@@ -1169,19 +1252,50 @@ struct partial_unwrap< Op< subview_col<eT>, op_htrans2> >
   
   inline
   partial_unwrap(const Op< subview_col<eT>, op_htrans2>& A)
-    : val( A.aux )
-    , M  ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, false, false )
+    : orig( A.m.m )
+    , val ( A.aux )
+    , M   ( const_cast<eT*>( A.m.colptr(0) ), A.m.n_rows, false, false )
     {
     arma_extra_debug_sigprint();
     }
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&orig)); }
+  
+  static const bool do_trans = true;
+  static const bool do_times = true;
+  
+  const Mat<eT>& orig;
+  
+  const eT      val;
+  const Col<eT> M;
+  };
+
+
+
+template<typename eT>
+struct partial_unwrap< Op< subview_row<eT>, op_htrans2> >
+  {
+  typedef Row<eT> stored_type;
+  
+  inline
+  partial_unwrap(const Op< subview_row<eT>, op_htrans2>& A)
+    : val(A.aux)
+    , M  (A.m  )
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  inline eT get_val() const { return val; }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
   static const bool do_trans = true;
   static const bool do_times = true;
   
   const eT      val;
-  const Col<eT> M;
+  const Row<eT> M;
   };
 
 
@@ -1202,6 +1316,8 @@ struct partial_unwrap_scalar_times_default
   
   arma_hot arma_inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1227,6 +1343,8 @@ struct partial_unwrap_scalar_times_fixed
   
   arma_hot arma_inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1275,6 +1393,8 @@ struct partial_unwrap< eOp<Mat<eT>, eop_scalar_times> >
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1299,6 +1419,8 @@ struct partial_unwrap< eOp<Row<eT>, eop_scalar_times> >
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1323,6 +1445,8 @@ struct partial_unwrap< eOp<Col<eT>, eop_scalar_times> >
   
   inline eT get_val() const { return val; }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1332,7 +1456,58 @@ struct partial_unwrap< eOp<Col<eT>, eop_scalar_times> >
 
 
 
-// TODO: struct partial_unwrap< eOp<subview_col<eT>, eop_scalar_times> >
+template<typename eT>
+struct partial_unwrap< eOp<subview_col<eT>, eop_scalar_times> >
+  {
+  typedef Col<eT> stored_type;
+  
+  arma_hot inline
+  partial_unwrap(const eOp<subview_col<eT>,eop_scalar_times>& A)
+    : orig( A.P.Q.m )
+    , val ( A.aux   )
+    , M   ( const_cast<eT*>( A.P.Q.colptr(0) ), A.P.Q.n_rows, false, false )
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_hot arma_inline eT get_val() const { return val; }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&orig)); }
+  
+  static const bool do_trans = false;
+  static const bool do_times = true;
+  
+  const Mat<eT>& orig;
+  
+  const eT      val;
+  const Col<eT> M;
+  };
+
+
+
+template<typename eT>
+struct partial_unwrap< eOp<subview_row<eT>, eop_scalar_times> >
+  {
+  typedef Row<eT> stored_type;
+  
+  arma_hot inline
+  partial_unwrap(const eOp<subview_row<eT>,eop_scalar_times>& A)
+    : val(A.aux)
+    , M  (A.P.Q)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_hot arma_inline eT get_val() const { return val; }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
+  static const bool do_trans = false;
+  static const bool do_times = true;
+  
+  const eT      val;
+  const Row<eT> M;
+  };
 
 
 
@@ -1351,6 +1526,8 @@ struct partial_unwrap_neg_default
   
   arma_hot arma_inline eT get_val() const { return eT(-1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1374,6 +1551,8 @@ struct partial_unwrap_neg_fixed
   
   arma_hot arma_inline eT get_val() const { return eT(-1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1420,6 +1599,8 @@ struct partial_unwrap< eOp<Mat<eT>, eop_neg> >
   
   inline eT get_val() const { return eT(-1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1442,6 +1623,8 @@ struct partial_unwrap< eOp<Row<eT>, eop_neg> >
   
   inline eT get_val() const { return eT(-1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1464,6 +1647,8 @@ struct partial_unwrap< eOp<Col<eT>, eop_neg> >
   
   inline eT get_val() const { return eT(-1); }
   
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&M)); }
+  
   static const bool do_trans = false;
   static const bool do_times = true;
   
@@ -1472,7 +1657,53 @@ struct partial_unwrap< eOp<Col<eT>, eop_neg> >
 
 
 
-// TODO: struct partial_unwrap< eOp<subview_col<eT>, eop_neg> >
+template<typename eT>
+struct partial_unwrap< eOp<subview_col<eT>, eop_neg> >
+  {
+  typedef Col<eT> stored_type;
+  
+  inline
+  partial_unwrap(const eOp<subview_col<eT>,eop_neg>& A)
+    : orig( A.P.Q.m )
+    , M   ( const_cast<eT*>( A.P.Q.colptr(0) ), A.P.Q.n_rows, false, false )
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_hot arma_inline eT get_val() const { return eT(-1); }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>& X) const { return (void_ptr(&X) == void_ptr(&orig)); }
+  
+  static const bool do_trans = false;
+  static const bool do_times = true;
+  
+  const Mat<eT>& orig;
+  const Col<eT>  M;
+  };
+
+
+
+template<typename eT>
+struct partial_unwrap< eOp<subview_row<eT>, eop_neg> >
+  {
+  typedef Row<eT> stored_type;
+  
+  inline
+  partial_unwrap(const eOp<subview_row<eT>,eop_neg>& A)
+    : M(A.P.Q)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_hot arma_inline eT get_val() const { return eT(-1); }
+  
+  arma_hot arma_inline bool is_alias(const Mat<eT>&) const { return false; }
+  
+  static const bool do_trans = false;
+  static const bool do_times = true;
+  
+  const Row<eT> M;
+  };
 
 
 
